/*
 * @(#)src/classes/sov/javax/swing/text/html/parser/ParserDelegator.java, swing, as142 1.13.2.2
 * ===========================================================================
 * Licensed Materials - Property of IBM
 * "Restricted Materials of IBM"
 *
 * IBM SDK, Java(tm) 2 Technology Edition, v1.4.2
 * (C) Copyright IBM Corp. 2003, 2004. All Rights Reserved
 * ===========================================================================
 */

/*
 * ===========================================================================
 * Copyright 2003 Sun Microsystems, Inc. All rights reserved.
 * ===========================================================================
 */
/* 
 *
 * Change activity:
 *
 * Reason  Date   Origin    Description
 * ------  ----   ------    -------------------------------------------------- 
 * 63757  050903 slattery  Add IBM module header; 
 * 168545 170810 dashekar Porting Sun Security alert 6938813; 
 *
 * ===========================================================================
 * Module Information:
 * 
 * DESCRIPTION: IBM.WRITEME
 * ===========================================================================
 */

/*
 * @(#)ParserDelegator.java	1.13 03/01/23
 *
 */

package javax.swing.text.html.parser;


import sun.awt.AppContext;                       /*ibm@168545*/
import javax.swing.text.html.HTMLEditorKit;
import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.DataInputStream;
import java.io.ObjectInputStream;
import java.io.Reader;
import java.io.Serializable;

/**
 * Responsible for starting up a new DocumentParser
 * each time its parse method is invoked. Stores a
 * reference to the dtd.
 *
 * @author  Sunita Mani
 * @version 1.13, 01/23/03
 */

public class ParserDelegator extends HTMLEditorKit.Parser implements Serializable {

    private static final Object DTD_KEY = new Object();              /*ibm@168545*/

    protected static void setDefaultDTD() {
        AppContext appContext = AppContext.getAppContext();         /*ibm@168545*/
        DTD dtd = (DTD) appContext.get(DTD_KEY);                    /*ibm@168545*/ 
        if (dtd == null) {
            synchronized(ParserDelegator.class) {
                if (dtd == null) {
                    DTD _dtd = null;
                    // (PENDING) Hate having to hard code!
                    String nm = "html32";
                    try {
                        _dtd = DTD.getDTD(nm);
                    } catch (IOException e) {
                        // (PENDING) UGLY!
                        System.out.println("Throw an exception: could not get default dtd: " + nm);
                    }
                    _dtd = createDTD(_dtd, nm);
                    dtd = _dtd;
                    appContext.put(DTD_KEY, dtd);                      /*ibm@168545*/

                }
            }
        }
    }

    protected static DTD createDTD(DTD dtd, String name) {

	InputStream in = null;
	boolean debug = true;
	try {
	    String path = name + ".bdtd";
	    in = getResourceAsStream(path);
            if (in != null) {
                dtd.read(new DataInputStream(new BufferedInputStream(in)));
                dtd.putDTDHash(name, dtd);
	    }
        } catch (Exception e) {
            System.out.println(e);
        }
        return dtd;
    }


    public ParserDelegator() {
	    setDefaultDTD();
    }

    public void parse(Reader r, HTMLEditorKit.ParserCallback cb, boolean ignoreCharSet) throws IOException {
	new DocumentParser((DTD) AppContext.getAppContext().get(DTD_KEY)).parse(r, cb, ignoreCharSet);      /*ibm@168545*/
    }

    /**
     * Fetch a resource relative to the ParserDelegator classfile.
     * If this is called on 1.2 the loading will occur under the
     * protection of a doPrivileged call to allow the ParserDelegator
     * to function when used in an applet.
     *
     * @param name the name of the resource, relative to the
     *  ParserDelegator class.
     * @returns a stream representing the resource
     */
    static InputStream getResourceAsStream(String name) {
	try {
            return ResourceLoader.getResourceAsStream(name);
	} catch (Throwable e) {
	    // If the class doesn't exist or we have some other 
	    // problem we just try to call getResourceAsStream directly.
	    return ParserDelegator.class.getResourceAsStream(name);
	}
    }

    private void readObject(ObjectInputStream s)
	throws ClassNotFoundException, IOException {
	s.defaultReadObject();
	
	    setDefaultDTD();
	
    }
}


